@@ -1,5 +1,5 @@
 """
- * $Id: 0.9+6 utils.py 88227807c1f8 2020-04-02 od $
+ * $Id: 0.9+7 utils.py 25e913d62457 2020-04-03 od $
  *
  * This file is part of the Cloud Services Integration Platform (CSIP),
  * a Model-as-a-Service framework, API and application suite.
@@ -14,12 +14,13 @@ from os import path
 from pathlib import Path
 from shutil import make_archive
 from time import sleep
-from typing import List, Dict
+from typing import List, Dict, Tuple
 
 import json
 import os
 import re
 import requests
+import sys
 
 
 class Client(object):
@@ -74,8 +75,11 @@ class Client(object):
         # the parent CSIP payload (request)
         self.parent = parent
 
-        # default delay for pubsub submission, default none
-        self.pubsub_delay = 0
+        # p/s delay for staggering pubsub submission, default none
+        self._delay = 0
+
+        # p/s batch subset to run, default all
+        self._batch = (1, sys.maxsize)
 
         if self.KEY_PATH in self.metainfo:
             self.metainfo[self.META_KEY_SERVICE_URL] = self.metainfo[self.KEY_PATH]
@@ -186,8 +190,8 @@ class Client(object):
 
         if not hasattr(self, "webhook"):
             raise Exception("No webhook provided.")
-        if not hasattr(self, "batch"):
-            raise Exception("No batch of files provided.")
+        if not hasattr(self, "files"):
+            raise Exception("No files provided.")
 
         h = {Client.HDR_CSIP_WEBHOOK: self.webhook}
 
@@ -195,12 +199,17 @@ class Client(object):
             h["Authorization"] = 'Bearer ' + self.bearertoken
 
         for i, file in enumerate(glob.iglob(self._files)):
-            h[Client.HDR_CSIP_REQUESTFILE] = Path(file).name
-            c = Client.from_file(file)
-            resp = c.execute(url, True, None, h)
-            callback(resp, file, i)
-            if self.pubsub_delay > 0:
-                sleep(self.pubsub_delay / 1000)
+            if i + 1 < self._batch[0]:  # before min
+                continue
+            if i + 1 <= self._batch[1]:  # within range
+                h[Client.HDR_CSIP_REQUESTFILE] = Path(file).name
+                c = Client.from_file(file)
+                resp = c.execute(url, True, None, h)
+                callback(resp, file, i)
+                if self._delay > 0:
+                    sleep(self._delay / 1000)
+            else:  # after max, we are done
+                break
 
     def execute(self, url: str = None, sync: bool = True, files: List[str] = None, headers: Dict = None) -> 'Client':
         """Executes a service as HTTP/POST. This method returns when service is finished if sync
@@ -370,24 +379,35 @@ class Client(object):
         self._webhook = hook
 
     @property
-    def batch(self) -> str:
+    def files(self) -> str:
         """For pubsub"""
         return self._files
 
-    @batch.setter
-    def batch(self, files: str) -> None:
+    @files.setter
+    def files(self, f: str) -> None:
         """For pubsub"""
-        self._files = files
+        self._files = f
+
+    @property
+    def batch(self) -> Tuple:
+        """For pubsub"""
+        return self._batch
+
+    @batch.setter
+    def batch(self, b: tuple) -> None:
+        """For pubsub"""
+        assert b[0] < b[1], "Invalid batch range"
+        self._batch = b
 
     @property
     def delay(self) -> int:
         """Get the delay between p/s submissions in ms"""
-        return self.pubsub_delay
+        return self._delay
 
     @delay.setter
     def delay(self, d: int) -> None:
         """Set the delay between p/s submissions in ms"""
-        self.pubsub_delay = d
+        self._delay = d
 
     #### metainfo
 
